package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.common.utils.StringUtil;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.*;
import cn.wisenergy.model.dto.UsersInfoDto;
import cn.wisenergy.model.vo.UserVo;
import cn.wisenergy.service.app.UserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author 86187
 * @ Description: 用户接口实现
 * @ Author     : 86187
 * @ Date       : 2021/1/6 16:11
 */
@Service
@Slf4j
public class UserServiceImpl extends ServiceImpl<UsersMapper, Users> implements UserService {

    @Resource
    private UsersMapper usersMapper;

    @Resource
    private ActivesMapper activesMapper;

    @Resource
    private UserRechargeMapper userRechargeMapper;

    @Resource
    private UserWithdrawsMapper withdrawsMapper;

    @Resource
    private UsdtOrderDetailsMapper orderDetailsMapper;

    @Resource
    private UserDataMapper userDataMapper;

    @Override
    public Users getById(Long userId) {
        return usersMapper.getById(userId);
    }

    @Override
    public R<UserVo> getLastUser(Long userId) {
        UserVo userVo = new UserVo();
        //用户id为空时，查询最顶级
        if (null == userId) {
            userId = 1L;
        }
        Users users = usersMapper.getById(userId);
        userVo.setUserId(users.getId());
        userVo.setParentId(users.getParentId());

        //获取用户统计数据
        UserData userData = userDataMapper.getByUserId(userId);
        userVo.setRealName(userData.getRealName());
        if (null != userData.getBuyTotal()) {
            userVo.setBuyTotal(userData.getBuyTotal().toString());
        }
        userVo.setBottom(userData.getBottom());
        userVo.setIdNumber(userData.getIdNumber());
        userVo.setPhone(userData.getPhone());
        userVo.setRank(userData.getRank());
        if (null != userData.getSaleTotal()) {
            userVo.setSaleTotal(userData.getSaleTotal().toString());
        }
        userVo.setTotalPeople(userData.getTotalPeople());

        if (null != userData.getTotalRecharge()) {
            userVo.setTotalRecharge(userData.getTotalRecharge().toString());
        }

        if (null != userData.getTotalWithdrawal()) {
            userVo.setTotalWithdrawal(userData.getTotalWithdrawal().toString());
        }

        //查询当前用户的下一级
        Users user = usersMapper.getById(userId);
        List<UserVo> lastList = usersMapper.getLastUser(Integer.valueOf(user.getId().toString()));
        userVo.setChildren(lastList);
        return R.ok(userVo);
    }

    @Override
    public R<UsersInfoDto> getByUserInfoById(Long userId) {
        if (null == userId) {
            return R.error("入参不能为空!");
        }

        UsersInfoDto userDto = new UsersInfoDto();

        //1、获取用户信息
        UserData user = userDataMapper.getByUserId(userId);
        if (null != user) {
            userDto.setUserId(user.getUserId());
            userDto.setPhone(user.getPhone());
            userDto.setRank(user.getRank());
            userDto.setIdNumber(user.getIdNumber());
            userDto.setRealName(user.getRealName());
            userDto.setBottom(user.getRank());
            userDto.setTotalPeople(user.getTotalPeople());
            userDto.setTotalWithdrawal(String.valueOf(user.getTotalWithdrawal()));
            userDto.setBuyTotal(String.valueOf(user.getBuyTotal()));
            userDto.setSaleTotal(String.valueOf(user.getSaleTotal()));
        }
        return R.ok(userDto);
    }

    @Override
    public R<List<UserVo>> getLinkTree(Long userId) {
        if (null == userId) {
            return R.error("入参不能为空!");
        }

        List<UserVo> userVos = new ArrayList<>();
        //获取用户信息
        Users user = usersMapper.getById(userId);
        if (null != user && StringUtils.isNotBlank(user.getPath())) {
            String path = user.getPath();
            List<Long> userIds = StringUtil.strToLongArray(path);

            //获取用户信息列表
            List<UserVo> list = usersMapper.getUserData(userIds);

            //递归生成树结构
            if (CollectionUtils.isNotEmpty(list)) {
                userVos = getChildren(list);
            }
        }
        return R.ok(userVos);
    }

    @Override
    public R<List<UserVo>> getThreeUser() {

        List<UserVo> userVos = new ArrayList<>();

        //获取前三级用户信息
        List<UserVo> list = usersMapper.getThreeUser();
        //递归生成树结构
        if (CollectionUtils.isNotEmpty(list)) {
            userVos = getChildren(list);
        }
        return R.ok(userVos);
    }

    private List<UserVo> getChildren(List<UserVo> list) {
        List<UserVo> rootList = new ArrayList<>();
        List<UserVo> childrenList = new ArrayList<>();

        for (UserVo userVo : list) {
            if (userVo.getParentId() == 0) {
                rootList.add(userVo);
            } else {
                childrenList.add(userVo);
            }
        }

        //遍历子数组，设置子树
        for (UserVo userVo : rootList) {
            List<UserVo> children = setChildren(userVo.getUserId(), childrenList);
            userVo.setChildren(children);
        }
        return rootList;
    }

    private List<UserVo> setChildren(Long userId, List<UserVo> list) {

        List<UserVo> children = new ArrayList<>();
        for (UserVo userVo : list) {
            Long parentId = userVo.getParentId().longValue();
            if (userId.equals(parentId)) {
                children.add(userVo);
            }
        }

        for (int i = 0; i < children.size(); i++) {
            Long autoId = children.get(i).getUserId();
            List<UserVo> childrenList = setChildren(autoId, list);
            children.get(i).setChildren(childrenList);
        }
        return children;
    }

}
